                 INCLUDE "VCS.I"
                 INCLUDE "MAKROS.I"

                 ORG $F000

SPRITE1_SIZE            EQU 9
ANIM_PHASES             EQU 3


ram_start               EQU $80
PlayfieldY              EQU ram_start
sprite1_aphase          EQU PlayfieldY + 1
sprite1_counter         EQU sprite1_aphase + 1
sprite1_y_pos           EQU sprite1_counter + 1
sprite1_x_pos_raw       EQU sprite1_y_pos + 1
sprite1_x_pos_fine      EQU sprite1_x_pos_raw + 1
sprite1_space           EQU sprite1_x_pos_fine + 1
screen_pos              EQU sprite1_space + SPRITE1_SIZE

Start:
                 CART_INIT
; clear all TIA registers except 0 (VSYNC)
                 LDA    #0
clear_memory_loop:
                 STA    0,X
                 DEX
                 BNE    clear_memory_loop

; init all needed variables...
                 JSR    GameInit
;
; Here is a representation of our program flow.
;
MainLoop:
                 JSR    VerticalBlank      ; Execute the vertical blank.
                 JSR    CheckSwitches      ; Check console switches.
                 JSR    GameCalc           ; Do calculations during Vblank
                 JSR    DrawScreen         ; Draw the screen
                 JSR    OverScan           ; Do more calculations during overscan
                 JMP    MainLoop           ; Continue forever.

;
; GameInit could conceivably be called when the Select key is pressed,
; or some other event.
;
GameInit:
                 LDA    #0
                 STA    PlayfieldY

; set up ports for input
                 STA    SWACNT             ; clearing a bit means DDR set for
                                           ; input, here all input bits

                 LDA    #100
                 STA    sprite1_y_pos

                 LDA    #9

                 STA    sprite1_x_pos_raw
                 LDA    #0
                 STA    sprite1_x_pos_fine

                 RTS
;
; It is important to maintain a stable screen, and this routine
; does some important and mysterious things.  Actually, the only
; mysterious part is VSYNC.  All VBLANK does is blank the TIA's
; output so that no graphics are drawn; otherwise the screen
; scans normally.  It is VSYNC which tells the TV to pack its
; bags and move to the other corner of the screen.
;
; Fortunately, my program sets VBLANK at the beginning of the
; overscan period, which usually precedes this subroutine, so
; it is not changed here.
;
VerticalBlank:
                  LDX   #0
                  LDA   #2
                  STA   WSYNC
                  STA   WSYNC
                  STA   WSYNC
                  STA   VSYNC              ; Begin vertical sync.
                  STA   WSYNC              ; First line of VSYNC
                  STA   WSYNC              ; Second line of VSYNC.
;
; But before we finish off the third line of VSYNC, why don't we
; use this time to set the timer?  This will save us a few cycles
; which would be more useful in the overscan area.
;
; To insure that we begin to draw the screen at the proper time,
; we must set the timer to go off just slightly before the end of
; the vertical blank space, so that we can WSYNC up to the ACTUAL
; end of the vertical blank space.  Of course, the scanline we're
; going to omit is the same scanline we were about to waste VSYNCing,
; so it all evens out.
;
; Atari says we have to have 37 scanlines of VBLANK time.  Since
; each scanline uses 76 cycles, that makes 37*76=2888 cycles.
; We must also subtract the five cycles it will take to set the
; timer, and the three cycles it will take to STA WSYNC to the next
; line.  Plus the checking loop is only accurate to six cycles, making
; a total of fourteen cycles we have to waste.  2888-14=2876.
;
; We almost always use TIM64T for this, since the math just won't
; work out with the other intervals.  2880/64=44.something.  It
; doesn't matter what that something is, we have to round DOWN.
;
                 LDA    #44                ; 44 times 64 cycles
                 STA    TIM64T             ; PIA timer start
;
; And now's as good a time as any to clear the collision latches.
;
                 LDA    #0
                 STA    CXCLR
;
; Now we can end the VSYNC period.
;
                 STA     WSYNC             ; Third line of VSYNC.
                 STA     VSYNC             ; (0)
;
; At this point in time the screen is scanning normally, but
; the TIA's output is suppressed.  It will begin again once
; 0 is written back into VBLANK.
;
                RTS
;
; Checking the game switches is relatively simple.  Theoretically,
; some of it could be slipped between WSYNCs in the VBlank code.
; But we're going for clarity here.
;
; It just so happens that I'm not going to check any game switches
; here.  I'm just going to set up the colors, without even checking
; the B&W switch!  HA!
;
CheckSwitches:
                 LDA    #0
                 STA    COLUBK             ; Background will be black.
                 RTS
;
; Minimal game calculations, just to get the ball rolling.
;
GameCalc:
                 LDA    sprite1_aphase

                 ADC    #SPRITE1_SIZE      ; length of one sprite
                 CMP    #(SPRITE1_SIZE*ANIM_PHASES); up 3 * sprite
                 BNE    no_overflow
                 LDA    #0
no_overflow:
                 CLC
                 STA    sprite1_aphase
                 ADC    #SPRITE1_SIZE-1
                 TAX
                 LDY    #SPRITE1_SIZE-1
sprite_copy_loop:
                 LDA    Man1_a,X
                 STA    sprite1_space,Y
                 DEX
                 DEY
                 BPL    sprite_copy_loop
                 LDA    #SPRITE1_SIZE-1    ; plus length - 1
                 STA    sprite1_counter


                 LDA    SWCHA
                 ASL    A
                 BCS    not_moved_right
                 LDX    sprite1_x_pos_fine
                 DEX
                 CPX    #-9
                 BNE    right_move_done
                 LDY    sprite1_x_pos_raw
                 CPY    #13
                 BEQ    right_move_done1
                 LDX    #6
                 INC    sprite1_x_pos_raw
right_move_done1:
                 INX
right_move_done:
                 STX    sprite1_x_pos_fine
                 PHA
                 TXA
                 ASL    A
                 ASL    A
                 ASL    A
                 ASL    A
                 TAX
                 PLA
                 STX    HMP0
not_moved_right:
                 ASL    A
                 BCS    not_moved_left
                 LDX    sprite1_x_pos_fine
                 INX
                 CPX    #8
                 BNE    left_move_done
                 LDY    sprite1_x_pos_raw
                 CPY    #5
                 BEQ    left_move_done1
                 LDX    #-7
                 DEC    sprite1_x_pos_raw
left_move_done1:
                 DEX
left_move_done:
                 STX    sprite1_x_pos_fine
                 PHA
                 TXA
                 ASL    A
                 ASL    A
                 ASL    A
                 ASL    A
                 TAX
                 PLA
                 STX    HMP0
not_moved_left:
                 ASL    A
                 BCS    not_moved_down
                 LDX    sprite1_y_pos
                 CPX    #SPRITE1_SIZE+2
                 BEQ    not_moved_down
                 DEC    sprite1_y_pos
not_moved_down:
                 ASL    A
                 BCS    not_moved_up
                 LDX    sprite1_y_pos
                 CPX    #119
                 BEQ    not_moved_up
                 INC    sprite1_y_pos
not_moved_up:
                 RTS

;
; This is the scariest thing I've done all month.
;
DrawScreen:
                 LDA     #0
                 STA     CTRLPF

                 LDA     #$1f
                 STA     COLUP0

; the LDA operand to 0 or 1 to use the normal system.  The code in
; the scanning loop accounts for both systems.
;

                 LDA    INTIM              ; check if blanking is over
                 BNE    DrawScreen         ; not yet, than check again!



                 STA    WSYNC
                 STA    VBLANK             ; End the VBLANK period with a zero.
;
; Initialize some display variables.
;
        ; There aren't any display variables!
;
; I'm going to use the Y register to count scanlines this time.
; Realize that I am forfeiting the use of the Y register for this
; purpose, but DEC Zero Page takes five cycles as opposed to DEY's
; two, and LDA Zero Page takes three cycles as opposed to TYA's two.
;
; I'm using all 191 remaining scanlines after the WSYNC.  If you
; want less, or more, change the LDY line.
;
; This is a decremental loop, that is, Y starts at 191 and continues
; down to 0, as do all functions of Y such as memory locations, which
; is why the graphics at the end of this file are stored "bottom-up".
; In a way, one could say that's how the screen is drawn.  To turn this
; into an incremental loop, change the number to 255-191 (65) and change
; the DEY at the end ot the scanning loop to INY.
;
                 LDY     #37
top_screen:
                 STA     WSYNC
                 LDX     top_back_color,Y
                 STX     COLUBK
                 DEY
                 BNE     top_screen

                 LDY     #119
ScanLoop:

; WSYNC is placed BEFORE all of this action takes place.
                 LDA     screen1_color,Y
                 STA     COLUPF
                 STA     WSYNC             ; [ 3] 3
                 LDA     screen_pf0a,Y     ; [ 4] 7
                 STA     PF0               ; [ 3] 10
                 LDA     screen_pf1a,Y     ; [ 4] 14
                 STA     PF1               ; [ 3] 17
                 LDA     screen_pf2a,Y     ; [ 4] 21
                 STA     PF2               ; [ 3] 24

                 NOP
                 NOP
                 NOP
; 40


                 LDA     screen_pf0b,Y     ; [ 4]
                 STA     PF0               ; [ 3]
                 LDA     screen_pf1b,Y     ; [ 4]
                 STA     PF1               ; [ 3]
                 LDA     screen_pf2b,Y     ; [ 4]
                 STA     PF2               ; [ 3]

                 CPY     sprite1_y_pos     ; [ 3]
                 BEQ     draw_man
                 DEY
                 BNE     ScanLoop

                 LDA     #0
                 STA     PF0               ; [ 3] 10
                 STA     PF1               ; [ 3] 17
                 STA     PF2               ; [ 3] 24
                 LDY     #34
bottom_screen:
                 STA     WSYNC
                 LDX     bottom_back_color,Y
                 STX     COLUBK
                 DEY
                 BNE     bottom_screen
;
; Clear all registers here to prevent any possible bleeding.
;
                 LDA    #2
                 STA    WSYNC              ; Finish this scanline.
                 STA    VBLANK             ; Make TIA output invisible,
                 ; Now we need to worry about it bleeding when we turn
                 ; the TIA output back on.
                 ; Y is still zero.
                 STY    PF0
                 STY    PF1
                 STY    PF2
                 STY    GRP0
                 STY    GRP1
                 STY    ENAM0
                 STY    ENAM1
                 STY    ENABL
                 RTS
draw_man:
                 DEY
                 LDX     sprite1_counter

                 LDA     screen1_color,Y;[4]
                 STA     COLUPF ;[3]
                 BRA     qq
ScanLoop_draw_man:
; WSYNC is placed BEFORE all of this action takes place.


;                 NOP;[2]
;                 NOP
                 NOP
                 JMP q; [3]
q:
                 LDA     screen1_color,Y;[4]
                 STA     COLUPF ;[3]

                 LDA     sprite1_color,X
                 STA     COLUP0            ;
qq:
                 STA     WSYNC             ; [ 3] 3
                 LDA     screen_pf0a,Y     ; [ 4] 7
                 STA     PF0               ; [ 3] 10
                 LDA     screen_pf1a,Y     ; [ 4] 14
                 STA     PF1               ; [ 3] 17

                 LDA     sprite1_space,X
                 STA     GRP0              ;

                 LDA     screen_pf2a,Y     ; [ 4] 21
                 STA     PF2               ; [ 3] 24

                 LDA     screen_pf0b,Y     ; [ 4]
                 STA     PF0               ; [ 3]
                 LDA     screen_pf1b,Y     ; [ 4]
                 STA     PF1               ; [ 3]
                 LDA     screen_pf2b,Y     ; [ 4]
                 STA     PF2               ; [ 3]


                 DEY
                 DEX
;                 DEC     sprite1_counter
                 BPL     ScanLoop_draw_man
                 BRA     ScanLoop
;
; For the Overscan routine, one might take the time to process such
; things as collisions.  I, however, would rather waste a bunch of
; scanlines, since I haven't drawn any players yet.
;
OverScan:                                  ; We've got 30 scanlines to kill.
                 LDX    #28
KillLines:
                 STA    WSYNC
                 DEX
                 BNE    KillLines
; position sprite 1 X
                 LDX    sprite1_x_pos_raw
                 BEQ    no_sprite_loop
                 STA    WSYNC
sprite1_pos_loop:
                 DEX
                 BNE    sprite1_pos_loop
                 STA    RESP0
                 LDX    sprite1_x_pos_raw
                 STA    WSYNC
                 STA    HMOVE
                 RTS
no_sprite_loop:
                 STA    WSYNC
                 STA    RESP0
                 LDX    sprite1_x_pos_raw
                 STA    WSYNC
                 STA    HMOVE
                 RTS

;
; Graphics are placed so that the extra cycle in the PFData,X indexes
; is NEVER taken, by making sure it never has to index across a page
; boundary.  This way our cycle count holds true.
;

                 ORG    $F600
;
; This is the tricky part of drawing a playfield: actually
; drawing it.  Well, the display routine and all that binary
; math was a bit tricky, too, but still, listen up.
;
; Playfield data isn't stored the way most bitmaps are, even
; one-dimensional bitmaps.  We will use the left side of the
; screen only, knowing that the right side is either repeated
; or reflected from it.
;
; In PF0 and PF2, the most significant bit (bit 7) is on the RIGHT
; side.  In PF1, the most significant bit is on the LEFT side.  This
; means that relative to PF0 and PF2, PF1 has a reversed bit order.
; It's just really weird.
;
;    PF0  |     PF1       |      PF2
;  4 5 6 7|7 6 5 4 3 2 1 0|0 1 2 3 4 5 6 7
;
; This is important to remember when doing calculations on bytes intended
; for the PF registers.  Defender gives a good example of this.
;
; It will become necessary to write a program that makes this easier,
; because it is easy to become confused when dealing with this system.
;          Right NULL Left Right Right Left Right NULL Left Right Right Left

                  ORG    $F700
 INCLUDE "scrncol.I"
                  ORG    $F800
 INCLUDE "SCREEN1.i"
                 ORG    $F900

; sprite 8, 8 + 1
Man1_a: ; upside down!
                 DB     %00000000
                 DB     %00011011
                 DB     %00010100
                 DB     %10111001
                 DB     %10111010
                 DB     %01111100
                 DB     %00010000
                 DB     %00111000
                 DB     %00111000

Man2_a: ; upside down!
                 DB     %00000000
                 DB     %01101100
                 DB     %00101000
                 DB     %10111001
                 DB     %10111010
                 DB     %01111100
                 DB     %00010000
                 DB     %00111000
                 DB     %00111000

Man3_a: ; upside down!
                 DB     %00000000
                 DB     %11011000
                 DB     %00101000
                 DB     %10111001
                 DB     %10111010
                 DB     %01111100
                 DB     %00010000
                 DB     %00111000
                 DB     %00111000
sprite1_color:
                 DB $00,$FF,$22,$22,$26,$2A,$2C,$2E,$26,$2A
 INCLUDE "BACK.I"

                 ORG    $FFFC
; reset vectors
                 DW     Start
                 DW Start
